@@ -16,11 +16,12 @@ use POSIX ':sys_wait_h';	#for WNOHANG in waitpid
 #$SIG{CHLD} = sub { while (waitpid(-1, WNOHANG)>0) {} };
 
 my (@cmd_and_args,$file,$ChildPID,$WatchTag,$WatchTag2,$OUTPUTfh,@pidToKill,$Kill9);
-my $Error;
 my $CMDfh;
 my (%supported,$mplayer);
 my $SoftVolume;
 my $GainFactor=1;
+my $playcounter;
+my $EQon;
 
 $::PlayPacks{Play_mplayer}=1; #register the package
 
@@ -33,7 +34,7 @@ sub init
 	$mplayer ||= ::first { -x $_ } map $_.::SLASH.'mplayer',  split /:/, $ENV{PATH};
 
 	return unless $mplayer;
-	return bless {RG=>1},__PACKAGE__;	#FIXME RG should be 0 if replaygain tags are disabled or if not $SoftVolume
+	return bless {RG=>1,EQ=>1},__PACKAGE__;	#FIXME RG should be 0 if replaygain tags are disabled or if not $SoftVolume
 }
 
 sub supported_formats
@@ -59,23 +60,14 @@ sub VolInit
 	return Play_amixer::init();	#use methods from Play_amixer
 }
 
-sub Play
-{	(undef,$file,my$sec)=@_;
-	&Stop if $ChildPID;
-	$Error=undef;
-	#if ($ChildPID) { print $CMDfh "loadfile $file\n"; print $CMDfh "seek $sec 2\n" if $sec; return}
+sub launch_mplayer
+{	$playcounter=0;
 	#-nocache because when using the cache option it spawns a child process, which makes monitoring the mplayer process much harder
 	#-hr-mp3-seek to fix wrong time with some mp3s
-	@cmd_and_args=($mplayer,qw/-nocache -slave -novideo -nolirc -hr-mp3-seek -msglevel all=1:statusline=5/);
-	RG_set_options();
-	push @cmd_and_args, qw/-softvol -volume/, convertvolume($::Volume) if $SoftVolume;
-	warn "@cmd_and_args\n" if $::debug;
+	@cmd_and_args=($mplayer,qw#-nocache -idle -slave -novideo -nolirc -hr-mp3-seek -msglevel all=1:statusline=5:global=6 --input=nodefault-bindings:conf=/dev/null#);
+	push @cmd_and_args, qw/-softvol -volume 0 --softvol-max=100/ if $SoftVolume;
 	push @cmd_and_args,split / /,$::Options{mplayeroptions} if $::Options{mplayeroptions};
-	push @cmd_and_args,'-ss',$sec if $sec;
-	push @cmd_and_args,'-ac','ffwavpack' if $file=~m/\.wvc?$/;
-	push @cmd_and_args,'-ac','ffape' if $file=~m/\.ape$/;
-	push @cmd_and_args, '--',$file;
-
+	warn "@cmd_and_args\n" if $::debug;
 	pipe $OUTPUTfh,my$wfh;
 	pipe my($rfh),$CMDfh;
 	$ChildPID=fork;
@@ -91,14 +83,31 @@ sub Play
 	}
 	close $wfh; close $rfh;
 	$CMDfh->autoflush(1);
-	#print $CMDfh "LOAD $file\n";
-	#SkipTo(undef,$sec) if $sec;
-
 	$OUTPUTfh->blocking(0); #set non-blocking IO
-	warn "playing $file (pid=$ChildPID)\n" if $::Verbose;
 	$WatchTag= Glib::IO->add_watch(fileno($OUTPUTfh),'hup',\&_eos_cb);
 	$WatchTag2=Glib::IO->add_watch(fileno($OUTPUTfh),'in',\&_remotemsg);
 		#Glib::Timeout->add(500,\&_UpdateTime);
+	return 1;
+}
+
+sub Play
+{	(undef,$file,my$sec)=@_;
+	launch_mplayer() unless $ChildPID;
+	print $CMDfh "loadfile \"$file\"\n";
+	$EQon= $::Options{use_equalizer} ? 1 : 0;
+	print $CMDfh "af_add equalizer=$::Options{equalizer}\n" if $EQon;
+	$playcounter++;
+	RG_set_options();
+	SetVolume(undef,$::Volume) if $SoftVolume;
+	$sec = $sec ? $sec : 0;
+	SkipTo(undef,$sec);
+	warn "playing $file (pid=$ChildPID)\n" if $::Verbose;
+}
+
+sub handle_error
+{	my $error=shift;
+	::ErrorPlay($error,_("Command used :")."\n@cmd_and_args");
+	Stop();
 }
 
 sub _eos_cb
@@ -106,28 +115,53 @@ sub _eos_cb
 	_remotemsg();#parse last lines
 	#close $OUTPUTfh;
 	if ($ChildPID && $ChildPID==waitpid($ChildPID, WNOHANG))
-	{	$Error=_"Check your audio settings" if $?;
+	{	$error=_"Check your audio settings" if $?;
 	}
 	while (waitpid(-1, WNOHANG)>0) {}	#reap dead children
 	Glib::Source->remove($WatchTag);
 	Glib::Source->remove($WatchTag2);
 	$WatchTag=$WatchTag2=$ChildPID=undef;
-	if ($Error) { ::ErrorPlay($Error,_("Command used :")."\n@cmd_and_args"); }
-	::end_of_file();
+	handle_error($error) if $error;
 	return 1;
 }
 
+sub set_equalizer
+{	my (undef,$val)=@_;
+	return unless $ChildPID;
+	return if !$EQon && $val eq '0:0:0:0:0:0:0:0:0:0';
+	# should be able to do either af_add or af_cmdline, but for some reason af_add doesn't
+	# set the equalizer, so do either af_add+af_cmdline or af_cmdline
+	print $CMDfh "af_add equalizer $val\n" unless $EQon;
+	print $CMDfh "af_cmdline equalizer $val\n";
+	$EQon=1;
+}
+
+sub EQ_Get_Range
+{	return (-12,12,'dB');
+}
+sub EQ_Get_Hz
+{	my $i=$_[1];
+	# mplayer and GST equalizers use the same bands, but they are indicated differently
+	# mplayer docs list band center frequences, GST reports band start freqs. Using GST values here for consistency
+	my @bands=(qw/29Hz 59Hz 119Hz 237Hz 474Hz 947Hz 1.9kHz 3.8kHz 7.5kHz 15.0kHz/);
+	return $bands[$i];
+}
 
 sub _remotemsg
-{	my @lines=(<$OUTPUTfh>);
-	for (reverse @lines)
-	{	if (m#^A:\s*(\d+)\.(\d) #) { ::UpdateTime( $1+($2>=5?1:0) ); return 1 }
-		# check if known error message
-		$Error=$1 if	m#(Could not open/initialize audio device)#	||
-				m#(Failed to open .+)#				||
-				m#(Failed to recognize file format)#;
+{	for (<$OUTPUTfh>)
+	{	if ($playcounter==1 && m#^A:\s*(\d+)\.(\d) #) { ::UpdateTime( $1+($2>=5?1:0) ); next }
+		warn "mplayer:$_" if $::debug;
+		if ($playcounter>0 && m#^EOF code: \d#)
+		{	::end_of_file() unless --$playcounter;
+		}
+		if (	m#^(Could not open/initialize audio device)#	||
+			m#^(Failed to open .+)#				||
+			m#^(Failed to recognize file format)#
+		   )
+		{	handle_error($1);
+			return 1;
+		}
 	}
-	warn join("mplayer:$_",'',@lines) if $::debug;
 	return 1;
 }
 
@@ -217,11 +251,11 @@ sub convertvolume	#convert a linear volume to cubic volume scale and apply $Gain
 
 sub RG_set_options
 {	return unless $SoftVolume;
-	if (defined($::PlayingID) && $::Options{gst_use_replaygain})
+	if (defined($::PlayingID) && $::Options{use_replaygain})
 	{	my ($gain1,$gain2,$peak1,$peak2)= Songs::Get($::PlayingID, qw/replaygain_track_gain replaygain_album_gain replaygain_track_peak replaygain_album_peak/);
-		($gain1,$gain2,$peak1,$peak2)= ($gain2,$gain1,$peak1,$peak2) if $::Options{gst_rg_albummode};
-		my $gain= ::first { $_ ne '' } $gain1, $gain2, $::Options{gst_rg_fallback};
-		$gain+= $::Options{gst_rg_preamp};
+		($gain1,$gain2,$peak1,$peak2)= ($gain2,$gain1,$peak1,$peak2) if $::Options{rg_albummode};
+		my $gain= ::first { $_ ne '' } $gain1, $gain2, $::Options{rg_fallback};
+		$gain+= $::Options{rg_preamp};
 		$GainFactor= 10**($gain/20);
 		my $peak= $peak1 || $peak2 || 1;
 		my $invpeak= 1/$peak;
